/*****************************************************************

 Title: Standard Macro Collection for Cox regression
 Author: Kengo NAGASHIMA
 Version: 1.0.5
 Last Updated Date: 2019/02/08
 Recommended Environment: SAS 9.3 and maybe later

 - 2019/02/08 bug fix for complete case analysis
 - 2015/09/15 competing risk
 - 2014/12/18 bug fix
 - 2014/11/18 bug fix

*****************************************************************/

/* global macro variables */
%global data out time event censorvalue class var color;
option spool;



%macro varstr2data(var, out);
data &out.;
  string = "&var.";
  do until(position <= 0);
    count + 1;
    call scan(string, count, position, length, ' ');
    word = substrn(string, position, length);
    if word ^= "" then output;
  end;
  keep word;
run;
%mend varstr2data;



%macro vmatch(var1, var2, out);
%varstr2data(&var1., &out.);
%varstr2data(&var2., _tmp_);
data &out.;
  set &out.;
  sort = _N_;
proc sort data = &out.; by word;
proc sort data = _tmp_; by word;
data &out.;
  merge &out.(in = x) _tmp_(in = y);
  by word;
  if x then do;
    if y then match = 1;
    else match = 0;
    output;
  end;
proc sort data = &out. out = &out.(drop = sort); by sort;
proc datasets lib = work;
  delete _tmp_;
run; quit;
%mend vmatch;



%macro autodummy(data, var);
proc freq data = &data. noprint;
  table &var. / out = _tmp_;
data _null_;
  set _tmp_;
  call symput("K", compress(_N_));
  call symput("L", compress(_N_ - 1));
run;
data _tmp_;
  set _tmp_;
  %do i = 1 %to &K.;
    %do j = 1 %to &L.;
      if _N_ = &i. then do;
        %if %eval(&i.) = %eval(&j.+1) %then &var._&j. = 1;
        %else &var._&j. = 0;;
      end;
    %end;
  %end;
  drop count percent;
run;
proc sort data = &data.; by &var.;
proc sort data = _tmp_; by &var.;
data &data.;
  merge &data. _tmp_; by &var.;
run;
proc datasets lib = work;
  delete _tmp_;
run; quit;
%mend autodummy;



%macro phregInit();

ods graphics off;
ods escapechar = '^';

/* initialize color counter */
%if "&color." = "0" %then %let color = 2;
%else %let color = 0;
%let class =;
%let var =;

/* initialize output data */
proc datasets lib = work;
  delete &out.;
run; quit;

%mend phregInit;



%macro phtemplate(font, fontsize, nprint, xprint);
proc template;
define style Styles.MyJournal;
  parent = styles.Journal;
  style fonts /
    'docFont' = (&font., &fontsize.)
    'TitleFont' = (&font., &fontsize., bold)
    'StrongFont' = (&font., &fontsize.)
    'EmphasisFont' = (&font., &fontsize.)
    'HeadingFont' = (&font., &fontsize.)
    'FixedEmphasisFont' = (&font., &fontsize.)
    'FixedFont' = (&font., &fontsize.)
    'TitleFont2' = (&font., &fontsize.)
    'BatchFixedFont' = (&font., &fontsize.)
    'FixedStrongFont' = (&font., &fontsize.);
end;
define table &out.tmpl;
  column
    name  hru lclu uclu pu  hrm lclm uclm pm
      col last;
  cellstyle
    col = "1" & last = "0" as
      {background = cxFFFFFF borderwidth = 0.5pt bordercolor = cxFFFFFF
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "1" & last = "1" as
      {background = cxFFFFFF borderbottomwidth = 0.5pt borderbottomcolor = black
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "1" & last = "2" as
      {background = cxFFFFFF borderwidth = 0.5pt bordercolor = cxFFFFFF
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "2" & last = "0" as
      {background = cxFAFAFA borderwidth = 0.5pt bordercolor = cxFAFAFA
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "2" & last = "1" as
      {background = cxFAFAFA borderbottomwidth = 0.5pt borderbottomcolor = black
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt};
  /* label */
  define name;
    header = hname; just = left;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.) protectspecialchars = off
    };
    define header hname;
      text "Covariate"; just = center;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF tagattr = "type:String" font = (&font., &fontsize.)
      };
    end;
  end;
  /* univariate */
  define header luni;
    %if "&nprint." ^= "0" %then %do;
      text "Univariate analysis (^{style[font_style=Italic]n} = &nprint.)";
    %end;
    %else %do;
      text "Univariate analysis";
    %end;
    start = hru; end = pu;
    style = {
      bordertopwidth = 0.5pt borderbottomwidth = 0.5pt borderbottomcolor = black
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
    };
  end;
  define hru;
    header = lhru; just = right;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
    define header lhru;
      text "HR"; just = center; start = hru; end = hru;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
      };
    end;
  end;
  define lclu;
    print_headers = off; just = right;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
  end;
  define uclu;
    print_headers = off; just = right;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
  end;
  define header luclu;
    text "95% C.I."; start = lclu; end = uclu;
    style = {
      bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
    };
  end;
  define pu;
    header = lpu; just = right; format = PVALUE5.;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
    define header lpu;
      text " ^{style[font_style=Italic]P}-value"; just = center; start = pu; end = pu;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
      };
    end;
  end;
  /* multivariate */
  define header lmult;
    %if "&nprint." ^= "0" %then %do;
      text "Multivariate analysis (^{style[font_style=Italic]n} = &xprint.)";
    %end;
    %else %do;
      text "Multivariate analysis";
    %end;
    start = hrm; end = pm;
    style = {
      bordertopwidth = 0.5pt borderbottomwidth = 0.5pt borderbottomcolor = black
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
    };
  end;
  define hrm;
    header = lhrm; just = right;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
    define header lhrm;
      text "HR"; just = center; start = hrm; end = hrm;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
      };
    end;
  end;
  define lclm;
    print_headers = off; just = right;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
  end;
  define uclm;
    print_headers = off; just = right;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
  end;
  define header luclm;
    text "95% C.I."; start = lclm; end = uclm;
    style = {
      bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
    };
  end;
  define pm;
    header = lpm; just = right; format = PVALUE5.;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., &fontsize.)
    };
    define header lpm;
      text " ^{style[font_style=Italic]P}-value"; just = center; start = pm; end = pm;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 0.5pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., &fontsize.) tagattr = "type:String"
      };
    end;
  end;
  define footer foot;
    text " ";
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      background = cxFFFFFF borderbottomcolor = white bordertopcolor = white
    };
  end;
  define col; print = off; end;
  define last; print = off; end;
end;
run;

%mend phtemplate;



%macro phregOut(
  firth = 0, eventcode = 0, nprint = 1, footer = 1,
  font = "Times New Roman", fontsize = 3
);

data _vtmp0_;
  set &out.;
%vmatch(&var., &class., _vtmp1_);
data _null_;
  set _vtmp1_;
  call symput("M", compress(_N_));
run;
data _null_;
  set _vtmp0_;
  %do h = 1 %to &M.;
    if _N_ = &h. then call symput("lvar&h.", trim(name));
  %end;
run;
data _null_;
  set _vtmp1_;
  %do h = 1 %to &M.;
    if _N_ = &h. then call symput("wvar&h.", compress(word));
    if _N_ = &h. then call symput("mvar&h.", compress(match));
  %end;
run;

/* Univariate */
%do h = 1 %to &M.;
data _udat_;
  set &data.(keep = &time. &event. &&wvar&h.);
  if cmiss(of _ALL_) = 0;
data _udat_;
  set _udat_;
  where &time. > 0;
proc phreg data = _udat_;
  %if "&&mvar&h." = "1" %then
    class &&wvar&h. / param = ref ref = first order = internal;;
  model &time.*&event.(&censorvalue.) = &&wvar&h. /
    rl 
    %if "&eventcode." ^= "0" %then eventcode = &eventcode. ties = breslow;
    %else %if "&firth." = "1" %then ties = breslow firth;
    %else ties = exact;;
  ods output ParameterEstimates = _est_(drop =
    %if "&&mvar&h." = "1" %then ClassVal0; 
    Label
  );
run;
%let varfmt =;
%if "&&mvar&h." = "1" %then %do;
proc freq data = _udat_;
  table &&wvar&h. / out = _freq_;
data _null_;
  set sashelp.vcolumn;
  if upcase(libname) = upcase("WORK") &
     upcase(memtype) = upcase("DATA") &
     upcase(memname) = upcase("&data.") &
     upcase(name) = upcase("&&wvar&h.")
  then call symput("varfmt", trim(format));
data _freq_;
  length vname $100.;
  set _freq_;
  %if "&varfmt." = " " %then vname = cat('&nbsp;&nbsp;&nbsp;&nbsp;', trim(cat(&&wvar&h.)));
  %else vname = cat('&nbsp;&nbsp;&nbsp;&nbsp;', trim(put(&&wvar&h., &varfmt.)));;
  keep vname;
run;
data _dummy_;
  length hru $10.;
  parameter = "&&wvar&h.";
  hru = "Reference";
data _est_;
  set _dummy_ _est_;
data _est_;
  length vparameter $100.;
  merge _est_ _freq_;
  vparameter = parameter;
run;
%end;
%else %do;
data _est_;
  length vparameter vname $100. hru $10.;
  set _est_;
  vname = "";
  vparameter = parameter;
  hru = "";
run;
%end;
proc append data = _est_ out = _outuniv_ force;
proc datasets lib = work;
  delete _udat_ _est_ _dummy_ _freq_;
run; quit;
%end;
data _outuniv_;
  length lclu uclu $10.;
  retain seq 0;
  set _outuniv_;
  xby = _N_;
  if hru ^= "Reference" then do;
    seq = seq + 1;
    hru = compress(put(round(HazardRatio, 0.001), 10.3));
    lclu = compress(put(round(HRLowerCL, 0.001), 10.3));
    uclu = compress(put(round(HRUpperCL, 0.001), 10.3));
    pu = ProbChiSq;
    by = seq;
  end;
  else by = .;
  keep vparameter xby by vname hru lclu uclu pu;
run;

data _adat_;
  set &data.(keep = &time. &event. &var.);
  if cmiss(of _ALL_) = 0;
data _adat_;
  set _adat_;
  where &time. > 0;
%if "&nprint." = "1" %then %do;
data _null_;
  set &data.;
  call symput("nprint", compress(_N_));
data _null_;
  set _adat_;
  call symput("xprint", compress(_N_));
run;
%end;

/* Multivariate */
proc phreg data = _adat_;
  class &class. / param = ref ref = first order = internal;
  model &time.*&event.(&censorvalue.) = &var. /
    rl 
    %if "&eventcode." ^= "0" %then eventcode = &eventcode. ties = breslow;
    %else %if "&firth." = "1" %then ties = breslow firth;
    %else ties = exact;;
  ods output ParameterEstimates = _outmult_;
  %if "&eventcode." ^= "0" %then ods output FailureSummary = _cs_;
  %else ods output CensoredSummary = _cs_;;
run;
data _outmult_;
  length vparameter $100.;
  set _outmult_;
  hrm = compress(put(round(HazardRatio, 0.001), 10.3));
  lclm = compress(put(round(HRLowerCL, 0.001), 10.3));
  uclm = compress(put(round(HRUpperCL, 0.001), 10.3));
  pm = ProbChiSq;
  by = _N_;
  vparameter = parameter;
  keep vparameter by hrm lclm uclm pm;
run;

/* merge */
data _outscore_;
  set _outmult_ end = final;
  ptime = .; prank = .; plogtime = .; pkm = .;
  if end = final then output;
  output;
  keep ptime prank plogtime pkm;
data _outscorel_;
  set _outscore_ end = final;
  if final then do;
    %if "&footer." ^= "0" %then
      name = cat(put(date(), yymmdd10.), " ", put(time(), time.));;
    last = "2";
    col = "1";
    output;
  end;
data _outscore_;
  set _outscore_ end = final;
  by = _N_;
  if ^final then output;
proc sort data = _outuniv_; by by;
proc sort data = _outmult_; by by;
proc sort data = _outscore_; by by;
data &out.;
  merge _outuniv_ _outmult_ _outscore_;
  by by;
proc sort data = &out.; by xby;
data &out.;
  length pvar name $100. col $8.;
  retain pvar "";
  set &out.;
  blank = "";
  if hru = "Reference" then hrm = "Reference";
  %do h = 1 %to &M.;
    if upcase(vparameter) = upcase("&&wvar&h.") then do;
      %if "&color." = "2" %then col = "1";
      %else col = compress(mod(&h. - 1, 2) + 1);;
      %if "&&mvar&h." = "0" %then %do;
        name = "&&lvar&h.";
        output;
      %end;
      %else %do;
        if upcase(pvar) ^= upcase(vparameter) then do;
          name = "&&lvar&h.";
          t_hru = hru; t_lclu = lclu; t_uclu = uclu; t_pu = pu;
          t_hrm = hrm; t_lclm = lclm; t_uclm = uclm; t_pm = pm;
          t_ptime = ptime; t_prank = prank; t_plogtime = plogtime; t_pkm = pkm;
          hru = ""; lclu = ""; uclu = ""; pu = .;
          hrm = ""; lclm = ""; uclm = ""; pm = .;
          ptime = .; prank = .; plogtime = .; pkm = .;
          output;
          name = vname;
          hru = t_hru; lclu = t_lclu; uclu = t_uclu; pu = t_pu;
          hrm = t_hrm; lclm = t_lclm; uclm = t_uclm; pm = t_pm;
          ptime = t_ptime; prank = t_prank; plogtime = t_plogtime; pkm = t_pkm;
          output;
        end;
        else do;
          name = vname;
          output;
        end;
      %end;
    end;
  %end;
  pvar = vparameter;
  drop xby by pvar vname vparameter t_hru t_lclu t_uclu t_pu t_hrm t_lclm t_uclm t_pm
    t_ptime t_prank t_plogtime t_pkm;
data &out.;
  set &out. end = final;
  if final then last = "1";
  else last = "0";
data &out.;
  set &out. _outscorel_;
run;
data &out.;
  set &out.;
  drop ptime prank plogtime pkm;
run;

proc datasets lib = work;
  delete _adat_ _vtmp1_ _vtmp2_ _cs_ _est_ _vtmp0_ _outscorel_ _dummy_
    _freq_ _outuniv_ _outmult_ _outscore_;
run; quit;

/* template gen */
%phtemplate(&font., &fontsize., &nprint., &xprint.);

%mend phregOut;



%macro phregMake(avar, label, aclass = 1);

data _vtmp_;
  length name $100.;
  name = &label.;
  output;
proc append base = &out. data = _vtmp_;
run;
%if "&aclass." = "1" %then %do;
  %let class=&class. &avar.;
  %let var=&var. &avar.;
%end;
%else %do;
  %let var=&var. &avar.;
%end;

proc datasets lib = work;
  delete _vtmp_;
run; quit;

%mend phregMake;



%macro render(data); 
data _null_;
  set &data.;
  file print ods = (template = "&data.tmpl");
  put _ods_;
run;
%mend render;



%macro outputInit(file);
ods listing close;
ods results off;

ods tagsets.excelxp file = &file. style = MyJournal;
  title "";
  options missing = " ";
%mend outputInit;



%macro outputMake(out, sheet, title, abscolwidth="30,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7", cox = 0);
%if "&cox." = "1" %then %let abscolwidth = "30,8,8,8,8,8,8,8,8,1,8,8,8,8";
ods tagsets.excelxp options(
  absolute_column_width = &abscolwidth.
  row_heights = "22"
  sheet_name = &sheet.
  embedded_titles = "yes"
);
title &title.;
%render(&out.);
%mend outputMake;



%macro outputOut();
  options missing = ".";
ods tagsets.excelxp close;

ods results on;
ods listing;
%mend outputOut;
